from rdflib.graph import Graph
from rdflib.term import URIRef

DATA = """<http://example.com#C> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2002/07/owl#Class>.
<http://example.com#B> <http://www.w3.org/2000/01/rdf-schema#subClassOf> _:fIYNVPxd4.
<http://example.com#B> <http://www.w3.org/2000/01/rdf-schema#subClassOf> <http://example.com#A>.
<http://example.com#B> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2002/07/owl#Class>.
<http://example.com#p1> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2002/07/owl#ObjectProperty>.
<http://example.com#A> <http://www.w3.org/2002/07/owl#unionOf> _:fIYNVPxd3.
<http://example.com#A> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2002/07/owl#Class>.
_:fIYNVPxd4 <http://www.w3.org/2002/07/owl#allValuesFrom> <http://example.com#C>.
_:fIYNVPxd4 <http://www.w3.org/2002/07/owl#onProperty> <http://example.com#p1>.
_:fIYNVPxd4 <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/2002/07/owl#Restriction>.
_:fIYNVPxd3 <http://www.w3.org/1999/02/22-rdf-syntax-ns#first> <http://example.com#B>.
_:fIYNVPxd3 <http://www.w3.org/1999/02/22-rdf-syntax-ns#rest> <http://www.w3.org/1999/02/22-rdf-syntax-ns#nil>.
"""

DATA_FALSE_ELEMENT = """
<http://example.org/#ThreeMemberList> <http://www.w3.org/1999/02/22-rdf-syntax-ns#first> <http://example.org/#p> .
<http://example.org/#ThreeMemberList> <http://www.w3.org/1999/02/22-rdf-syntax-ns#rest> _:list2 .
_:list2 <http://www.w3.org/1999/02/22-rdf-syntax-ns#first> "false"^^<http://www.w3.org/2001/XMLSchema#boolean> .
_:list2 <http://www.w3.org/1999/02/22-rdf-syntax-ns#rest> _:list3 .
_:list3 <http://www.w3.org/1999/02/22-rdf-syntax-ns#first> <http://example.org/#r> .
_:list3 <http://www.w3.org/1999/02/22-rdf-syntax-ns#rest> <http://www.w3.org/1999/02/22-rdf-syntax-ns#nil> .
"""


class TestOWLCollectionTest:
    def test_collection_rdfxml(self):
        g = Graph().parse(data=DATA, format="nt")
        g.namespace_manager.bind("owl", URIRef("http://www.w3.org/2002/07/owl#"))
        s = g.serialize(format="pretty-xml")  # noqa: F841


class TestListTest:
    def test_false_element(self):
        g = Graph().parse(data=DATA_FALSE_ELEMENT, format="nt")
        assert len(list(g.items(URIRef("http://example.org/#ThreeMemberList")))) == 3



"""This runs the turtle tests for the W3C RDF Working Group's Turtle
test suite."""

from __future__ import annotations

import logging
from contextlib import ExitStack
from typing import Optional

import pytest

from rdflib.graph import Graph
from test.data import TEST_DATA_DIR
from test.utils import BNodeHandling, GraphHelper, ensure_suffix
from test.utils.dawg_manifest import ManifestEntry, params_from_sources
from test.utils.iri import URIMapper
from test.utils.namespace import RDFT

logger = logging.getLogger(__name__)

REMOTE_BASE_IRI = "http://www.w3.org/2013/TurtleTests/"
LOCAL_BASE_DIR = TEST_DATA_DIR / "suites/w3c/turtle/"
ENCODING = "utf-8"
MAPPER = URIMapper.from_mappings(
    (REMOTE_BASE_IRI, ensure_suffix(LOCAL_BASE_DIR.as_uri(), "/"))
)
VALID_TYPES = {
    RDFT.TestTurtlePositiveSyntax,
    RDFT.TestTurtleNegativeSyntax,
    RDFT.TestTurtleEval,
    RDFT.TestTurtleNegativeEval,
}

# NOTE: `rdft:TestTrigNegativeEval` entries does not have `mf:result` as suggested
# by <https://www.w3.org/TR/rdf11-testcases/#evaluation-tests>, instead they seem
# to be regular `rdft:TestTrigNegativeSyntax`. For example, see `trig-eval-bad-01`
# <https://www.w3.org/2013/TrigTests/manifest.ttl>


def check_entry(entry: ManifestEntry) -> None:
    assert entry.action is not None
    assert entry.type in VALID_TYPES
    action_path = entry.uri_mapper.to_local_path(entry.action)
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug(
            "action = %s\n%s", action_path, action_path.read_text(encoding=ENCODING)
        )
    catcher: Optional[pytest.ExceptionInfo[Exception]] = None
    graph = Graph()
    with ExitStack() as xstack:
        if entry.type in (RDFT.TestTurtleNegativeSyntax, RDFT.TestTurtleNegativeEval):
            catcher = xstack.enter_context(pytest.raises(Exception))
        graph.parse(action_path, publicID=entry.action, format="turtle")

    if catcher is not None:
        assert catcher.value is not None

    if entry.type == RDFT.TestTurtleEval:
        assert entry.result is not None
        result_source = entry.uri_mapper.to_local_path(entry.result)
        result_graph = Graph()
        result_graph.parse(result_source, publicID=entry.action, format="ntriples")
        GraphHelper.assert_isomorphic(graph, result_graph)
        GraphHelper.assert_sets_equals(
            graph, result_graph, bnode_handling=BNodeHandling.COLLAPSE
        )


MARK_DICT = {
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-LITERAL2_with_langtag_and_datatype": pytest.mark.xfail(
        reason="accepts literal with both language and datatype"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-uri-01": pytest.mark.xfail(
        reason="accepts invalid IRIs"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-uri-02": pytest.mark.xfail(
        reason="accepts invalid IRIs"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-uri-03": pytest.mark.xfail(
        reason="accepts invalid IRIs"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-uri-04": pytest.mark.xfail(
        reason="accepts invalid IRIs"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-uri-05": pytest.mark.xfail(
        reason="accepts invalid IRIs"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-04": pytest.mark.xfail(
        reason="accepts literal as subject"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-05": pytest.mark.xfail(
        reason="accepts literal as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-06": pytest.mark.xfail(
        reason="accepts blank node as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-07": pytest.mark.xfail(
        reason="accepts blank node as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-kw-04": pytest.mark.xfail(
        reason="accepts 'true' as subject"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-kw-05": pytest.mark.xfail(
        reason="accepts 'true' as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-n3-extras-03": pytest.mark.xfail(
        reason="accepts N3 paths"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-n3-extras-04": pytest.mark.xfail(
        reason="accepts N3 paths"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-n3-extras-06": pytest.mark.xfail(
        reason="accepts N3 paths"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-14": pytest.mark.xfail(
        reason="accepts Literal as subject"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-15": pytest.mark.xfail(
        reason="accepts Literal as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-16": pytest.mark.xfail(
        reason="accepts blank node as as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-struct-17": pytest.mark.xfail(
        reason="accepts blank node as as predicate"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-esc-02": pytest.mark.xfail(
        reason="accepts badly escaped literal"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-esc-03": pytest.mark.xfail(
        reason="accepts badly escaped literal"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-esc-04": pytest.mark.xfail(
        reason="accepts badly escaped literal"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-string-06": pytest.mark.xfail(
        reason="accepts badly quoted string"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-string-07": pytest.mark.xfail(
        reason="accepts badly quoted string"
    ),
    f"{REMOTE_BASE_IRI}#turtle-eval-bad-01": pytest.mark.xfail(
        reason="accepts invalid IRI"
    ),
    f"{REMOTE_BASE_IRI}#turtle-eval-bad-02": pytest.mark.xfail(
        reason="accepts invalid IRI"
    ),
    f"{REMOTE_BASE_IRI}#turtle-eval-bad-03": pytest.mark.xfail(
        reason="accepts invalid IRI"
    ),
    f"{REMOTE_BASE_IRI}#turtle-eval-bad-04": pytest.mark.xfail(
        reason="accepts invalid IRI"
    ),
    f"{REMOTE_BASE_IRI}#turtle-syntax-bad-ln-dash-start": pytest.mark.xfail(
        reason="accepts dash at the start of local name production"
    ),
}


@pytest.mark.parametrize(
    ["manifest_entry"],
    params_from_sources(
        MAPPER,
        ManifestEntry,
        LOCAL_BASE_DIR / "manifest.ttl",
        mark_dict=MARK_DICT,
        report_prefix="rdflib_w3c_turtle",
    ),
)
def test_entry(manifest_entry: ManifestEntry) -> None:
    check_entry(manifest_entry)
